@@ -60,7 +60,7 @@ const Dashboard = ({
             title={t('refresh_btn')}
             onClick={() => getAllStats()}
     >
-        <svg className="icons">
+        <svg className="icons icon12">
             <use xlinkHref="#refresh" />
         </svg>
     </button>;
